/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE aoTuvWriter SOURCE CODE.                *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2007             *
 * by the Xiph.Org Foundation http://www.xiph.org/                  *
 *                                                                  *
 * THE aoTuv SOURCE CODE IS (C) COPYRIGHT 2003-2008 Aoyumi          *
 * http://www.geocities.jp/aoyoume/aotuv/                           *
 *                                                                  *
 * THE aoTuvWriter SOURCE CODE IS (C) COPYRIGHT 2008 Cocha          *
 * http://sourceforge.jp/projects/directshow/                       *
 *                                                                  *
 ********************************************************************/

#include "aoTuvWriter.h"

// ------------------------------------------------------------------------------------------------------------------------
STDMETHODIMP CInputPin::Receive(IMediaSample *pSample)
{
   HRESULT hr;

   CheckPointer(pSample, E_POINTER);
   CAutoLock lock(m_pReceiveLock);

   REFERENCE_TIME tStart, tStop;
   pSample->GetTime(&tStart, &tStop);

   m_tLast = tStart;

   BYTE *pbData;
   hr = pSample->GetPointer(&pbData);
   if(FAILED(hr))
      return hr;

   hr = paoTuvEncode->Receive((short *)pbData, pSample->GetActualDataLength() / 2);

   return hr;
}
// ------------------------------------------------------------------------------------------------------------------------
STDMETHODIMP CInputPin::EndOfStream(void)
{  // 󂯎f[^ȂꍇɌĂ΂

   CAutoLock lock(m_pReceiveLock);
   CAutoLock cObjectLock(m_pLock);

   if(paoTuvEncode == NULL)
      return E_FAIL;

   paoTuvEncode->EncodeEnd();

   return CRenderedInputPin::EndOfStream();
}
// ------------------------------------------------------------------------------------------------------------------------
HRESULT CInputPin::Run(REFERENCE_TIME tStart)
{  // tB^sꂽꍇɌĂ΂

   CAutoLock lock(m_pReceiveLock);
   CAutoLock cObjectLock(m_pLock);

   // łɃt@C݂ȂAt@C폜
   if(::PathFileExists(m_awFileName) != FALSE)
   {
      if(::DeleteFile(m_awFileName) == FALSE)
         return E_FAIL; // t@C폜Ɏs
   }

   m_VorbisFormat.nChannels = m_InputWaveFormatEx.nChannels;
   m_VorbisFormat.nSamplesPerSec = m_InputWaveFormatEx.nSamplesPerSec;

   HRESULT hr;
   hr = paoTuvEncode->EncodeInit(m_awFileName, &m_VorbisFormat, &m_VorbisTag);
   if(FAILED(hr))
      return hr;

   return S_OK;
}
// ------------------------------------------------------------------------------------------------------------------------
HRESULT CInputPin::Stop()
{  // ~{^ꂽꍇɌĂ΂

   CAutoLock lock(m_pReceiveLock);
   CAutoLock cObjectLock(m_pLock);

   if(paoTuvEncode == NULL)
      return E_FAIL;

   paoTuvEncode->EncodeEnd();

   return S_OK;
}
// ------------------------------------------------------------------------------------------------------------------------
HRESULT CInputPin::CheckMediaType(const CMediaType *pMediaType)
{  // ̓sق̃sƌqɌĂ΂

   CAutoLock lock(m_pReceiveLock);

   if(*pMediaType->Subtype() == MEDIASUBTYPE_PCM  &&  *pMediaType->FormatType() == FORMAT_WaveFormatEx)
   {
      if( ((WAVEFORMATEX *)pMediaType->Format())->wFormatTag != WAVE_FORMAT_PCM)
      {  // kPCMȊOȂ󂯕tȂ
         return S_FALSE;
      }

      if( ((WAVEFORMATEX *)pMediaType->Format())->wBitsPerSample != 16)
      {  // 16bitȊOȂ󂯕tȂ
         return S_FALSE;
      }

      if( ((WAVEFORMATEX *)pMediaType->Format())->nSamplesPerSec > 48000)
      {   // TvOg48kHz傫Ȃ󂯕tȂ
         return S_FALSE;
      }

      if( ((WAVEFORMATEX *)pMediaType->Format())->nChannels != 1 && ((WAVEFORMATEX *)pMediaType->Format())->nChannels != 2)
      {   // mXeIȊOȂȂ󂯕tȂ
         return S_FALSE;
      }

      // ͂̉i[Ă
      ::CopyMemory(&m_InputWaveFormatEx, pMediaType->Format(), sizeof(WAVEFORMATEX));

      return S_OK;
   }

   return S_FALSE;
}
// -----------------------------------------------------------------------------------------------------------------------------------
HRESULT CInputPin::BreakConnect()
{
   if(m_pFilter->m_pPosition != NULL)
      m_pFilter->m_pPosition->ForceRefresh();

   return CRenderedInputPin::BreakConnect();
}
// -----------------------------------------------------------------------------------------------------------------------------------
HRESULT CInputPin::SetVorbisFormat(VORBISFORMAT *pVorbisFormat)
{
   int ret;

   // vorbis̏ɐ邩Ă݂
   vorbis_info vi;
   vorbis_info_init(&vi);

   if(-0.11f <= pVorbisFormat->fQuality && pVorbisFormat->fQuality <= 1.01f)
   {  // VBȐ
      ret = vorbis_encode_init_vbr(&vi, (long)m_InputWaveFormatEx.nChannels, m_InputWaveFormatEx.nSamplesPerSec, pVorbisFormat->fQuality);
   }
   else
   {  // ABR,CBȐ
      ret = vorbis_encode_init(&vi, (long)m_InputWaveFormatEx.nChannels, m_InputWaveFormatEx.nSamplesPerSec, pVorbisFormat->nMaxBitsPerSec, pVorbisFormat->nAvgBitsPerSec, pVorbisFormat->nMinBitsPerSec);
   }

   if(ret != 0)
   {  // Ɏsꍇ
      return E_FAIL;
   }

   vorbis_info_clear(&vi);


   // ݒi[Ă
   ::CopyMemory(&m_VorbisFormat, pVorbisFormat, sizeof(VORBISFORMAT));

   return S_OK;
}
// -----------------------------------------------------------------------------------------------------------------------------------
HRESULT CInputPin::SetVorbisTag(VORBISTAG *pVorbisTag)
{
   // ݒi[Ă
   ::CopyMemory(&m_VorbisTag, pVorbisTag, sizeof(VORBISTAG));

   return S_OK;
}
// -----------------------------------------------------------------------------------------------------------------------------------
HRESULT CInputPin::SetFileName(LPCOLESTR pszFileName)
{
   // t@Ci[Ă
   lstrcpyW(m_awFileName, pszFileName);

   return S_OK;
}
// -----------------------------------------------------------------------------------------------------------------------------------
CInputPin::CInputPin(CaoTuvWriter *pFilter, LPUNKNOWN pUnk, CCritSec *pLock, CCritSec *pReceiveLock, HRESULT *phr)
   : CRenderedInputPin(NAME("Input"), pFilter, pLock, phr, L"Input"), m_pReceiveLock(pReceiveLock), m_pFilter(pFilter), m_tLast(0)
{  // CInputPiñRXgN^

   paoTuvEncode = new CaoTuvEncode();

   // ϐ̏
   ::ZeroMemory(m_awFileName, MAX_PATH);
   ::ZeroMemory(&m_VorbisFormat, sizeof(VORBISFORMAT));
   ::ZeroMemory(&m_VorbisTag, sizeof(VORBISTAG));
}
// ------------------------------------------------------------------------------------------------------------------------
CInputPin::~CInputPin()
{  // CInputPiñfXgN^

   if(paoTuvEncode != NULL)
      delete paoTuvEncode;
}
// ------------------------------------------------------------------------------------------------------------------------
